# references the local flake of the Nebelhorn project
nebelhornFlake:

/*
on the other hand, these refer to the flake where this module is imported
i.e. any particular site's flake
*/
{ self, lib, flake-parts-lib, ... }:

{
  options.perSystem = flake-parts-lib.mkPerSystemOption ({ ... }: {
    options.nebelhorn = {
      enable = lib.mkEnableOption "Enable Nebelhorn support";
      name = lib.mkOption {
        description = "The name of the website package to build.";
        default = "example.com";
        type = lib.types.str;
      };
      projectPath = lib.mkOption {
        description = "The path containing the Nebelhorn project.";
        default = self;
        defaultText = "The path of the flake.";
        type = lib.types.path;
      };
      config = lib.mkOption {
        description = "The nebelhorn config as verbatim YAML. Will use a local `nebelhorn.yaml` if not specified.";
        default = null;
        type = lib.types.nullOr (lib.types.lines);
      };
    };
  });

  config.perSystem = { pkgs, system, config, ... }: lib.mkIf config.nebelhorn.enable {

    packages.default = pkgs.stdenvNoCC.mkDerivation {
      name = config.nebelhorn.name;
      src = config.nebelhorn.projectPath;
      buildInputs = [
        (nebelhornFlake.withSystem system ({ self', ... }: self'.packages.nebelhorn))
        pkgs.sassc
      ];
      phases = [ "unpackPhase" "installPhase" ];
      installPhase = if config.nebelhorn.config != null
        then ''
          nebelhorn -o $out -c ${pkgs.writeText "nebelhorn.yaml" config.nebelhorn.config}
        ''
        else ''
          nebelhorn -o $out
        '';
    };

    apps.default = {
      type = "app";
      program = pkgs.writeShellApplication {
        name = config.nebelhorn.name;
        runtimeInputs = [ pkgs.haskellPackages.wai-app-static ];
        text = ''
          warp -d result
        '';
      };
    };

  };
}
