{
  inputs = {
    nixpkgs.url = "github:nixos/nixpkgs/nixpkgs-unstable";
    flake-parts.url = "github:hercules-ci/flake-parts";
    haskell-flake.url = "github:srid/haskell-flake";
    systems.url = "github:nix-systems/default";
  };

  outputs = inputs: inputs.flake-parts.lib.mkFlake { inherit inputs; } (
    { flake-parts-lib, withSystem, ... }: {

      systems = import inputs.systems;
      imports = [ inputs.haskell-flake.flakeModule ];

      perSystem = { self', pkgs, ... }: {

        haskellProjects.default = {
          packages = {
            tztime.source = "0.1.0.0";
          };
          devShell = {
            mkShellArgs = {
              nativeBuildInputs = [ pkgs.sassc ];
            };
          };
        };

        packages.default = pkgs.haskell.lib.justStaticExecutables self'.packages.nebelhorn;
        apps.default = self'.apps.nebelhorn;

        packages.image = pkgs.dockerTools.buildImage {
          name = "nebelhorn";
          copyToRoot = pkgs.buildEnv {
            name = "image-root";
            paths = [ pkgs.sassc ];
            pathsToLink = [ "/bin" ];
          };
          config = {
            Cmd = [ "${self'.packages.default}/bin/nebelhorn" ];
          };
        };

      };

      flake.flakeModules.default = flake-parts-lib.importApply ./flake-module.nix {
        inherit withSystem;
      };

    }
  );
}
