module Config (Config(..), ConfigSource(..), ConfigFeed(..), ConfigCopy(..), ConfigFilter(..), loadConfig) where

import Relude
import Data.Yaml
import Path
import Data.Time.TZInfo (TZIdentifier)

-- | A nebelhorn config.
data Config = Config {
    configBaseUrl :: Text,
    configSources :: [ConfigSource],
    configFeeds :: [ConfigFeed],
    configCopyItems :: [ConfigCopy],
    configFilters :: [ConfigFilter],
    configTimezone :: TZIdentifier
} deriving (Show)

data ConfigSource = ConfigSource {
    configSourceSource :: FilePath,
    configSourceOutput :: FilePath,
    configSourceTemplate :: Path Rel File,
    configSourceDefaultCategories :: [Text]
} deriving (Show)

data ConfigFeed = ConfigFeed {
    configFeedTitle :: Text,
    configFeedOutput :: Path Rel File,
    configFeedCategories :: [Text]
} deriving (Show)

data ConfigCopy = ConfigCopy {
    configCopySource :: FilePath,
    configCopyOutput :: FilePath
} deriving (Show)

newtype ConfigFilter = ConfigFilter {
    configFilterFile :: Path Rel File
    -- TODO support applyTo, is currently applied to EVERYTHING
} deriving (Show)

instance FromJSON Config where
    -- The somewhat opaque "v .:? foo .!= bar" syntax means:
    -- Get the attribute named foo out of v, if it doesn't exist, use bar as default value
    parseJSON = withObject "Config" $ \v -> Config
        <$> v .:? "baseUrl" .!= "http://localhost:3000"
        <*> v .:? "sources" .!= []
        <*> v .:? "feeds" .!= []
        <*> v .:? "copyItems" .!= []
        <*> v .:? "filters" .!= []
        <*> v .:? "timezone" .!= ""

instance FromJSON ConfigSource where
    parseJSON = withObject "ConfigSource" $ \v -> ConfigSource
        <$> v .: "source"
        <*> v .: "output"
        <*> v .: "template"
        <*> v .:? "defaultCategories" .!= []

instance FromJSON ConfigFeed where
    parseJSON = withObject "ConfigFeed" $ \v -> ConfigFeed
        <$> v .:? "title" .!= "Untitled Feed"
        <*> v .: "output"
        <*> v .: "categories"

instance FromJSON ConfigCopy where
    parseJSON = withObject "ConfigCopy" $ \v -> ConfigCopy
        <$> v .: "source"
        <*> v .: "output"

instance FromJSON ConfigFilter where
    parseJSON = withObject "ConfigFilter" $ \v -> ConfigFilter
        <$> v .: "file"

-- | Loads a config file from the nebelhorn.yaml in the current directory.
loadConfig :: FilePath -> IO (Either ParseException Config)
loadConfig = decodeFileEither
