{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE TypeFamilies #-}

module Main (main) where

import Relude
import Config
import Build
import Options.Applicative
import Path
import Path.IO

data CliOptions = CliOptions
    { cliOptionsOutput :: String
    , cliOptionsConfigFile :: String
    } deriving Show

parser :: Parser CliOptions
parser = CliOptions
    <$> strOption
        ( long "output"
        <> short 'o'
        <> help "Where to write the output"
        <> showDefault
        <> value "out"
        <> metavar "PATH"
        )
    <*> strOption
        ( long "config-file"
        <> short 'c'
        <> help "The config file to use"
        <> showDefault
        <> value "nebelhorn.yaml"
        <> metavar "CONFIG_FILE"
        )

-- | The main function. Loads the config and calls build.
main :: IO ()
main = run =<< execParser opts where
    opts = info (parser <**> helper)
        ( fullDesc
        <> progDesc "Build a static Nebelhorn site"
        <> header "Nebelhorn - a static site generator based on Pandoc"
        )

run :: CliOptions -> IO ()
run CliOptions {..} = do
    config' <- loadConfig cliOptionsConfigFile
    case (config', parseSomeDir cliOptionsOutput) of
        (Left err, _) -> print err
        (_, Left err) -> print err
        (Right config, Right outputPath') -> do
            -- only needed until path-io 1.8
            -- which has an instance of AnyPath for SomeBase
            outputPath <- case outputPath' of
                Abs a -> makeAbsolute a
                Rel a -> makeAbsolute a
            build config outputPath
            putStrLn "Success!"
