{-# LANGUAGE QuasiQuotes #-}

module Save (save, copyItems, copyItem) where

import Relude
import Path
import Path.IO
import System.Process
import Text.Pandoc.Writers.HTML
import Text.Pandoc.Class
import Text.Pandoc.Options
import Text.Pandoc.Highlighting
import Nebelhorn
import Feed
import Util
import Control.Concurrent.Async

-- | Saves a Nebelhorn.
save :: Path Abs Dir -> Nebelhorn -> IO ()
save outputPath nebelhorn@Nebelhorn{..} = do
    writeDocument outputPath `mapConcurrently_` nebelhornPages
    writeFeeds outputPath nebelhorn
    putStrLn "Written Feeds"
    ensureDir $ outputPath </> [reldir|style|]
    liftIO $ callCommand $ "sassc --sourcemap=auto --style compressed "
        <> nebelhornStylesheet <> " "
        <> toFilePath (outputPath </> [relfile|style/stylesheet.css|])
    liftIO $ copyItems outputPath nebelhornCopyItems

-- | Saves a list of files to the specified paths relative to a base path.
writeDocument :: Path b Dir -> Page -> IO ()
writeDocument basePath page = do
    let fileOutput = basePath </> page.pageTarget
    putStrLn $ "writing " <> toFilePath fileOutput
    ensureDir $ parent fileOutput
    let writerOptions = def { writerExtensions = pandocExtensions, writerTemplate = Just page.pageTemplate, writerHighlightStyle = Just breezeDark}
    output <- runIOorExplode (writeHtml5String writerOptions page.pageDoc)
    writeFileTextUtf8 (toFilePath fileOutput) output
    putStrLn $ "Written " <> toFilePath fileOutput


-- | Copies folders verbatim according to the target folder.
copyItems :: Path Abs Dir -> [CopyItem] -> IO ()
copyItems outputPath items = forM_ items (copyItem outputPath)

copyItem :: Path Abs Dir -> CopyItem -> IO ()
copyItem outputPath CopyItemDir {..} = copyDirRecur copyItemDirSource (outputPath </> copyItemDirTarget)
copyItem outputPath CopyItemFile {..} = copyFile copyItemFileSource (outputPath </> copyItemFileTarget)
